

import java.io.FileInputStream;   
import java.io.InputStreamReader;   
import java.io.BufferedReader;  

class IORParser
{
	private class BinaryParser
	{
		byte buf[];
		int ptr = 0;
		boolean bigEndian;

		BinaryParser (byte buf[])
		{
			this.buf = buf;
			this.ptr = 0;
			this.bigEndian = true;
		}

		int getOctet()
		{
			int value = buf[ptr];
			if (value < 0)
			{
				value += 256;
			}
			ptr++;
			return value;
		}

		byte[] getOctets(int num)
		{
			byte j[] = new byte[num];

			for (int i=0; i < num; i++)
			{
				j[i] = (byte) getOctet();
			}
			return j;
		}

		void align(int size)
		{
			int numAlignBytes = size - (ptr % size);

			if (numAlignBytes == size)
			{
				numAlignBytes = 0;
			}

			ptr += numAlignBytes;
		}

		int getInt32()
		{
			align(4);

			long value;

			if (bigEndian)
			{
				value =  getOctet() * 0x1000000;
				value += getOctet() * 0x10000;
				value += getOctet() * 0x100;
				value += getOctet();
			}
			else
			{
				value =  getOctet();
				value += getOctet() * 0x100;
				value += getOctet() * 0x10000;
				value += getOctet() * 0x1000000;
			}

			return (int) value;
		}

		int getInt16()
		{
			align(2);

			int value;

			if (bigEndian)
			{
				value =  getOctet() * 0x100;
				value += getOctet();
			}
			else
			{
				value =  getOctet();
				value += getOctet() * 0x100;
			}

			return value;
		}

		String getString()
		{
			String s = null;
			int len = getInt32();

			try
			{
			  s = new String (buf, ptr, len, "ascii");
			}
			catch (java.io.UnsupportedEncodingException e)
			{
				e.printStackTrace();
			}

			getOctets(len);
			return s;
		}


	}


	private class TaggedComponent
	{
		TaggedComponent (int type, byte b[])
		{
		  // noch nicht fertig...
		}
	}

	private class IIOPProfile
	{
		int versionMajor;
		int versionMinor;
		String host;
		int port;
		byte objectKey[];
		TaggedComponent components[];


		IIOPProfile(byte b[])
			throws IllegalArgumentException
		{
			BinaryParser p = new BinaryParser(b);

			int x = p.getOctet();
			if (x == 0)
			{
				p.bigEndian = true;
			}
			else if (x == 1)
			{
				p.bigEndian = false;
			}
			else
			{
				throw new IllegalArgumentException(
				  "first octet not 0 or 1 (endian detection)");
			}


			versionMajor = p.getOctet();

			if (versionMajor != 1)
			{
			  throw new IllegalArgumentException(
			  	"IIOP version not supported: " + versionMajor);
			}

			versionMinor = p.getOctet();
			host = p.getString();
			port = p.getInt16();
			int len = p.getInt32();
			objectKey = p.getOctets(len);

			if (versionMinor >= 1)
			{
			  int num =  p.getInt32();
			  components = new TaggedComponent[num];

			  for (int i=0; i < num; i++)
			  {
			    int type = p.getInt32();
			    len = p.getInt32();
			    components[i] = new TaggedComponent(type,
			    	p.getOctets(len));
			  }
			}

		}

		void printInfo()
		{
		    System.out.println(" IIOP Version: " + versionMajor +
		       "." + versionMinor);

		    System.out.println(" Hostname: " + host);
		    System.out.println(" Port: " + port);
		    System.out.print(" ObjectKey:");

		    for (int i=0; i < objectKey.length; i++)
		    {
		    	int value = objectKey[i];
			if (value < 0)
			{
				value += 256;
			}
		    	String n = Integer.toHexString(value);
			if (n.length() < 2)
			{
				n = "0" + n;
			}
		    	System.out.print(" " + n);
		    }

		    System.out.println();

		}
	}
	

	private String typeID;
	private BinaryParser binParser;
	private IIOPProfile profiles[];

	public IORParser (String ior)
		throws IllegalArgumentException
	{
		if (! ior.toLowerCase().startsWith("ior:"))
		{
			throw new IllegalArgumentException(
			  "IOR-String doesn't start with 'IOR:'");
		}

		if ((ior.length() % 2) != 0)
		{
			throw new IllegalArgumentException(
			  "HexString has odd number of digits");
		}

		byte iorBytes[] = new byte[(ior.length() - 4) / 2];
		
		int i;

		for (i=4; i < ior.length(); i+=2)
		{
			iorBytes[(i - 4) / 2] =
			(byte)	Integer.parseInt(ior.substring(i, i+2), 16);
		}

		binParser = new BinaryParser(iorBytes);

		int x = binParser.getOctet();

		if (x == 0)
		{
			binParser.bigEndian = true;
		}
		else if (x == 1)
		{
			binParser.bigEndian = false;
		}
		else
		{
			throw new IllegalArgumentException(
			  "IOR first octet not 0 or 1 (endian detection)");
		}

		typeID = binParser.getString();


		int numProfiles = binParser.getInt32();


		profiles = new IIOPProfile[numProfiles];

		for (i = 0; i < numProfiles; i++)
		{
			int profileID = binParser.getInt32();
			int numBytes = binParser.getInt32();


			byte k[] = binParser.getOctets(numBytes);

			if (profileID == 0)
			{
				profiles[i] = new IIOPProfile(k);
			}
			else
			{
			throw new IllegalArgumentException(
			  "unsupported IOR profile ID " + profileID);
			}
		}
	}

	public void printInfo()
	{
		System.out.println ("typeID: " + typeID);

		for (int i=0; i < profiles.length; i++)
		{
			profiles[i].printInfo();
		}
	}


	public static void main (String argv[])
	{
		try {
			BufferedReader b = new BufferedReader(
				new InputStreamReader(
				new FileInputStream(argv[0]))); 

			IORParser p = new IORParser(b.readLine());

			p.printInfo();

		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
}
